// import {
//     EcdsaVerifier,
//     EcdsaWasmLoader,
//     RSAVerifier,
//     RSAWasmLoader,
//     verifySignatureByParam
// } from './verifier' 

// @ts-ignore
import CryptoJSW from '@originjs/crypto-js-wasm'
import crypto from 'crypto'
import { env } from "../../internals"
import { BasicSignature } from './signature'
// @ts-ignore
import NodeRSA from 'node-rsa'
// @ts-ignore
import ecdsa from 'ecdsa-wasm'
// @ts-ignore
import eccrypto from '@comtstore/eccrypto'


/**
 * 
 * @param {{
*   publicKey: string,
*   type: number,
*   bit: number,
*   webAssembly: boolean,
*   signal: string,
*   content: string,
*   rsaWasmLoaderInstance: RSAWasmLoader,
*   rsaVerifierInstance: RSAVerifier,
*   ecdsaWasmLoaderInstance: EcdsaWasmLoader,
*   ecdsaVerifierInstance: EcdsaVerifier
* }} options 
* @returns {boolean}
*/
export const verifySignatureByParam = (options) => {
  const { publicKey, type, bit, webAssembly, signal, content, 
   rsaWasmLoaderInstance, rsaVerifierInstance,
   ecdsaWasmLoaderInstance, ecdsaVerifierInstance } = options
  /**
    * @type {boolean}
    */
  let verifyResult = false
 
  if (type === 0) {
    const loadingState = rsaWasmLoaderInstance.getLoadingState()
    if (webAssembly && [0, 3].includes(loadingState)) {
     rsaWasmLoaderInstance.load()
    }
    if (webAssembly && loadingState === 2) {
      verifyResult = rsaWasmLoaderInstance.verify(signal, publicKey, content)
    } else {
      verifyResult = rsaVerifierInstance.verify(signal, publicKey, content, bit)
    }
  } else if (type === 1) {
    try {
      const loadingState = ecdsaWasmLoaderInstance.getLoadingState()
      if (webAssembly && [0, 3].includes(loadingState)) {
        ecdsaWasmLoaderInstance.load()
      }
      if (webAssembly && loadingState === 2) {
        verifyResult = ecdsaWasmLoaderInstance.verify(signal, publicKey, content)
      } else {
        verifyResult = ecdsaVerifierInstance.verify(signal, publicKey, content)
      }
    } catch (err) {
      console.error(err)
    }
  }

  return verifyResult
}

export class EcdsaVerifier {
    /**
    * @type {EcdsaVerifier}
    */
    static instance
 
    /**
     * 获取实例
     * @returns {EcdsaVerifier} 
     */
    static getInstance(){
      if(!EcdsaVerifier.instance){
       EcdsaVerifier.instance = new EcdsaVerifier()
      }
      return EcdsaVerifier.instance
    }
 
    /**
     * 
     * @param {string} signal 
     * @param {string} privateKey 
     * @returns 
     */
    sign(signal, privateKey){
      return JSON.stringify(eccrypto.signSync(
        Buffer.from(JSON.parse(privateKey).data),
        crypto.createHash('sha256').update(signal).digest()
      ))
    }

    /**
     * 
     * @param {string} signal 
     * @param {string} publicKey 
     * @param {BasicSignature | string} signature 
     * @returns 
     */
    verify(signal, publicKey, signature){
      let content = typeof signature === 'string' ? signature : signature.content
      return eccrypto.verifySync(
        Buffer.from(JSON.parse(publicKey).data),
        crypto.createHash('sha256').update(signal).digest(),
        Buffer.from(JSON.parse(content).data)
      )  
    }
 }
 
export class RSAVerifier {
   /**
      * @type {RSAVerifier}
      */
   static instance
 
   /**
     * 获取实例
     * @returns {RSAVerifier} 
     */
   static getInstance(){
     if(!RSAVerifier.instance){
       RSAVerifier.instance = new RSAVerifier()
     }
     return RSAVerifier.instance
   }

   /**
    * 
    * @param {string} signal 
    * @param {string} privateKey 
    * @param {number} bit 
    * @returns 
    */
   sign(signal, privateKey, bit){
    const key = new NodeRSA({ b: bit }, {
      environment: env.getEnv() === 'node' ? 'node' : 'browser'
    })
    key.importKey(privateKey, 'pkcs8-private-pem')
    return JSON.stringify(key.sign(Buffer.from(signal)))
   }
 
   /**
     * 
     * @param {string} signal 
     * @param {string} publicKey
     * @param {BasicSignature | string} signature 
     * @param {number} bit
     * @returns 
     */
   verify(signal, publicKey, signature, bit){
     let content = typeof signature === 'string' ? signature : signature.content
     const key = new NodeRSA({ b: bit }, {
       environment: env.getEnv() === 'node' ? 'node' : 'browser'
     })
     key.importKey(publicKey, 'pkcs8-public-pem')
     return key.verify(Buffer.from(signal), Buffer.from(JSON.parse(content).data)) 
   }
 }
 
export class EcdsaWasmLoader {
   /**
    * @type {EcdsaWasmLoader}
    */
   static instance
 
   /**
    * 获取实例
    * @returns {EcdsaWasmLoader} 
    */
   static getInstance(){
     if(!EcdsaWasmLoader.instance){
       EcdsaWasmLoader.instance = new EcdsaWasmLoader()
     }
     return EcdsaWasmLoader.instance
   }
 
   constructor(){
     this.loadingState = 0 // 0 = 未开始加载， 1 = 正在加载， 2 = 加载完成， 3 = 加载失败
   }
 
   getLoadingState(){
     return this.loadingState
   }
 
   load(){
     this.loadingState = 1
     ecdsa.init().then(() => {
       // ecdsa.setSerializeMode(ecdsa.SerializeOld)
       this.loadingState = 2
     }).catch(() => {
       this.loadingState = 3
     })
   }

   /**
    * 
    * @param {string} signal 
    * @param {string} privateKey 
    * @returns 
    */
   sign(signal, privateKey){
    const _privateKey = Buffer.from(JSON.parse(privateKey).data).toString('hex')
    try {
      const _sec = ecdsa.deserializeHexStrToSecretKey(_privateKey)
      const sig = JSON.stringify(Buffer.from(_sec.sign(signal).serializeToHexStr(), 'hex'))
      return sig
    } catch (err) {
      return ''
    }
   }
 
   /**
    * 
    * @param {string} signal 
    * @param {string} publicKey 
    * @param {BasicSignature | string} signature 
    * @returns {boolean}
    */
   verify (signal, publicKey, signature) {
    let content = typeof signature === 'string' ? signature : signature.content
    const _publicKey = Buffer.from(JSON.parse(publicKey).data).toString('hex')
    const _signature = Buffer.from(JSON.parse(content).data).toString('hex')
    try {
      const _pub = ecdsa.deserializeHexStrToPublicKey(_publicKey)
      const _sig = ecdsa.deserializeHexStrToSignature(_signature)
      return _pub.verify(_sig, signal)
    } catch (err) {
      return false
    }
   }
 }
 
export class RSAWasmLoader {
   /**
    * @type {RSAWasmLoader}
    */
   static instance
 
   /**
    * 获取实例
    * @returns {RSAWasmLoader} 
    */
   static getInstance(){
     if(!RSAWasmLoader.instance){
       RSAWasmLoader.instance = new RSAWasmLoader()
     }
     return RSAWasmLoader.instance
   }
 
   constructor(){
     this.loadingState = 0
   }
 
   getLoadingState(){
     return this.loadingState
   }
 
   load(){
     this.loadingState = 1
     CryptoJSW.RSA.loadWasm().then(() => {
       this.loadingState = 2
     }).catch(() => {
       this.loadingState = 3
     })
   }

   /**
    * 
    * @param {string} signal 
    * @param {string} privateKey 
    * @returns {string}
    */
   sign(signal, privateKey){
    const RSA = CryptoJSW.RSA
    return JSON.stringify(Buffer.from(
      RSA.sign(
        new Uint8Array(crypto.createHash('sha256').update(signal).digest()), {
          isPublicKey: false,
          key: privateKey,
          signPadding: 'PKCS1V15'
        }
      ).buffer))
   }
 
   /**
    * 
    * @param {string} signal 
    * @param {string} publicKey 
    * @param {BasicSignature | string} signature 
    * @returns 
    */
   verify(signal, publicKey, signature){
    let content = typeof signature === 'string' ? signature : signature.content
    const RSA = CryptoJSW.RSA
    const verifyResult = RSA.verify(
      new Uint8Array(crypto.createHash('sha256').update(signal).digest()),
      new Uint8Array(Buffer.from(JSON.parse(content).data)),
      {
        isPublicKey: true,
        key: publicKey,
        signPadding: 'PKCS1V15'
      }
    )
    return verifyResult
   }
 }
 
let rsaWasmLoaderInstance = new RSAWasmLoader()
let rsaVerifierInstance = new RSAVerifier()
let ecdsaWasmLoaderInstance = new EcdsaWasmLoader()
let ecdsaVerifierInstance = new EcdsaVerifier()

self.addEventListener('message', (e) => {
    const data = JSON.parse(e.data)
    const { publicKey, webAssembly, type, signal, content, bit, id } = data

    const verifyResult = verifySignatureByParam({
        publicKey,
        type,
        bit,
        webAssembly,
        signal,
        content,
        rsaWasmLoaderInstance,
        rsaVerifierInstance,
        ecdsaWasmLoaderInstance,
        ecdsaVerifierInstance
    })

    self.postMessage(JSON.stringify({
        result: verifyResult ? 1 : 0,
        id
    }))
})

export default {}